@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-"""
-Sphinx Auto-API Top-level Extension.
+"""Sphinx Auto-API Top-level Extension.
 
 This extension allows you to automagically generate API documentation from your project.
 """
@@ -8,10 +6,11 @@ import io
 import os
 import shutil
 import sys
+from typing import Dict, Tuple
 import warnings
 
 import sphinx
-from sphinx.util.console import darkgreen, bold
+from sphinx.util.console import colorize
 from sphinx.addnodes import toctree
 from sphinx.errors import ExtensionError
 import sphinx.util.logging
@@ -27,7 +26,6 @@ from .backends import (
 from .directives import AutoapiSummary, NestedParse
 from .inheritance_diagrams import AutoapiInheritanceDiagram
 from .settings import API_ROOT
-from .toctree import add_domain_to_toctree
 
 LOGGER = sphinx.util.logging.getLogger(__name__)
 
@@ -40,11 +38,8 @@ _DEFAULT_OPTIONS = [
     "special-members",
     "imported-members",
 ]
-_VIEWCODE_CACHE = {}
-"""Caches a module's parse results for use in viewcode.
-
-:type: dict(str, tuple)
-"""
+_VIEWCODE_CACHE: Dict[str, Tuple[str, Dict]] = {}
+"""Caches a module's parse results for use in viewcode."""
 
 
 class RemovedInAutoAPI2Warning(DeprecationWarning):
@@ -55,7 +50,7 @@ if "PYTHONWARNINGS" not in os.environ:
     warnings.filterwarnings("default", category=RemovedInAutoAPI2Warning)
 
 
-def _normalise_autoapi_dirs(autoapi_dirs, confdir):
+def _normalise_autoapi_dirs(autoapi_dirs, srcdir):
     normalised_dirs = []
 
     if isinstance(autoapi_dirs, str):
@@ -64,7 +59,7 @@ def _normalise_autoapi_dirs(autoapi_dirs, confdir):
         if os.path.isabs(path):
             normalised_dirs.append(path)
         else:
-            normalised_dirs.append(os.path.normpath(os.path.join(confdir, path)))
+            normalised_dirs.append(os.path.normpath(os.path.join(srcdir, path)))
 
     return normalised_dirs
 
@@ -74,13 +69,9 @@ def run_autoapi(app):  # pylint: disable=too-many-branches
     Load AutoAPI data from the filesystem.
     """
     if app.config.autoapi_type not in LANGUAGE_MAPPERS:
+        allowed = ", ".join(f'"{api_type}"' for api_type in sorted(LANGUAGE_MAPPERS))
         raise ExtensionError(
-            "Invalid autoapi_type setting, "
-            "following values is allowed: {}".format(
-                ", ".join(
-                    '"{}"'.format(api_type) for api_type in sorted(LANGUAGE_MAPPERS)
-                )
-            )
+            f"Invalid autoapi_type setting, following values are allowed: {allowed}"
         )
 
     if not app.config.autoapi_dirs:
@@ -96,12 +87,12 @@ def run_autoapi(app):  # pylint: disable=too-many-branches
             app.config.autoapi_options.append("show-module-summary")
 
     # Make sure the paths are full
-    normalised_dirs = _normalise_autoapi_dirs(app.config.autoapi_dirs, app.confdir)
+    normalised_dirs = _normalise_autoapi_dirs(app.config.autoapi_dirs, app.srcdir)
     for _dir in normalised_dirs:
         if not os.path.exists(_dir):
             raise ExtensionError(
-                "AutoAPI Directory `{dir}` not found. "
-                "Please check your `autoapi_dirs` setting.".format(dir=_dir)
+                f"AutoAPI Directory `{_dir}` not found. "
+                "Please check your `autoapi_dirs` setting."
             )
 
     normalized_root = os.path.normpath(
@@ -113,31 +104,24 @@ def run_autoapi(app):  # pylint: disable=too-many-branches
         import_name in sys.modules
         for _, import_name in LANGUAGE_REQUIREMENTS[app.config.autoapi_type]
     ):
+        packages = ", ".join(
+            f'{import_name} (available as "{pkg_name}" on PyPI)'
+            for pkg_name, import_name in LANGUAGE_REQUIREMENTS[app.config.autoapi_type]
+        )
         raise ExtensionError(
-            "AutoAPI of type `{type}` requires following "
-            "packages to be installed and included in extensions list: "
-            "{packages}".format(
-                type=app.config.autoapi_type,
-                packages=", ".join(
-                    '{import_name} (available as "{pkg_name}" on PyPI)'.format(
-                        pkg_name=pkg_name, import_name=import_name
-                    )
-                    for pkg_name, import_name in LANGUAGE_REQUIREMENTS[
-                        app.config.autoapi_type
-                    ]
-                ),
-            )
+            f"AutoAPI of type `{app.config.autoapi_type}` requires following "
+            f"packages to be installed and included in extensions list: {packages}"
         )
 
     sphinx_mapper = LANGUAGE_MAPPERS[app.config.autoapi_type]
     template_dir = app.config.autoapi_template_dir
     if template_dir and not os.path.isabs(template_dir):
         if not os.path.isdir(template_dir):
-            template_dir = os.path.join(app.confdir, app.config.autoapi_template_dir)
-        elif app.confdir != os.getcwd():
+            template_dir = os.path.join(app.srcdir, app.config.autoapi_template_dir)
+        elif app.srcdir != os.getcwd():
             warnings.warn(
                 "autoapi_template_dir will be expected to be "
-                " relative to conf.py instead of "
+                "relative to the Sphinx source directory instead of "
                 "relative to where sphinx-build is run\n",
                 RemovedInAutoAPI2Warning,
             )
@@ -159,7 +143,7 @@ def run_autoapi(app):  # pylint: disable=too-many-branches
         out_suffix = ".txt"
     else:
         # Fallback to first suffix listed
-        out_suffix = app.config.source_suffix[0]
+        out_suffix = next(iter(app.config.source_suffix))
 
     if sphinx_mapper_obj.load(
         patterns=file_patterns, dirs=normalised_dirs, ignore=ignore_patterns
@@ -173,10 +157,13 @@ def run_autoapi(app):  # pylint: disable=too-many-branches
 def build_finished(app, exception):
     if not app.config.autoapi_keep_files and app.config.autoapi_generate_api_docs:
         normalized_root = os.path.normpath(
-            os.path.join(app.confdir, app.config.autoapi_root)
+            os.path.join(app.srcdir, app.config.autoapi_root)
         )
         if app.verbosity > 1:
-            LOGGER.info(bold("[AutoAPI] ") + darkgreen("Cleaning generated .rst files"))
+            LOGGER.info(
+                colorize("bold", "[AutoAPI] ")
+                + colorize("darkgreen", "Cleaning generated .rst files")
+            )
         shutil.rmtree(normalized_root)
 
         sphinx_mapper = LANGUAGE_MAPPERS[app.config.autoapi_type]
@@ -195,13 +182,11 @@ def doctree_read(app, doctree):
     Inject AutoAPI into the TOC Tree dynamically.
     """
 
-    add_domain_to_toctree(app, doctree, app.env.docname)
-
     if app.env.docname == "index":
         all_docs = set()
         insert = True
         nodes = list(doctree.traverse(toctree))
-        toc_entry = "%s/index" % app.config.autoapi_root
+        toc_entry = f"{app.config.autoapi_root}/index"
         add_entry = (
             nodes
             and app.config.autoapi_generate_api_docs
@@ -219,11 +204,11 @@ def doctree_read(app, doctree):
                 insert = False
         if insert and app.config.autoapi_add_toctree_entry:
             # Insert AutoAPI index
-            nodes[-1]["entries"].append((None, u"%s/index" % app.config.autoapi_root))
-            nodes[-1]["includefiles"].append(u"%s/index" % app.config.autoapi_root)
-            message_prefix = bold("[AutoAPI] ")
-            message = darkgreen(
-                "Adding AutoAPI TOCTree [{0}] to index.rst".format(toc_entry)
+            nodes[-1]["entries"].append((None, f"{app.config.autoapi_root}/index"))
+            nodes[-1]["includefiles"].append(f"{app.config.autoapi_root}/index")
+            message_prefix = colorize("bold", "[AutoAPI] ")
+            message = colorize(
+                "darkgreen", f"Adding AutoAPI TOCTree [{toc_entry}] to index.rst"
             )
             LOGGER.info(message_prefix + message)
 
@@ -258,11 +243,12 @@ def viewcode_find(app, modname):
             stack.extend((full_name + ".", gchild) for gchild in children)
 
     if module.obj["encoding"]:
-        source = io.open(
-            module.obj["file_path"], encoding=module.obj["encoding"]
-        ).read()
+        stream = io.open(module.obj["file_path"], encoding=module.obj["encoding"])
     else:
-        source = open(module.obj["file_path"]).read()
+        stream = open(module.obj["file_path"], encoding="utf-8")
+
+    with stream as in_f:
+        source = in_f.read()
 
     result = (source, locations)
     _VIEWCODE_CACHE[modname] = result
@@ -270,7 +256,7 @@ def viewcode_find(app, modname):
 
 
 def viewcode_follow_imported(app, modname, attribute):
-    fullname = "{}.{}".format(modname, attribute)
+    fullname = f"{modname}.{attribute}"
     all_objects = app.env.autoapi_all_objects
     if fullname not in all_objects:
         return None
@@ -323,6 +309,6 @@ def setup(app):
     app.add_directive("autoapi-inheritance-diagram", AutoapiInheritanceDiagram)
 
     return {
-        "parallel_read_safe": False,
+        "parallel_read_safe": True,
         "parallel_write_safe": True,
     }
